/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.features;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.Iterator;
import java.util.LinkedList;

import org.apache.hadoop.io.Writable;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class SURFFeatures extends Features implements Writable{

	/**
	 * default constructor
	 */
	public SURFFeatures(){
		this.features = new LinkedList<SURFFeature>();
		this.ID = "";
	}
	
	/**
	 * 
	 */
	private static final long serialVersionUID = -2050374417865103264L;
	
	private String ID;
	
	public class SURFFeature extends Feature implements Serializable{

		/**
		 * 
		 */
		private static final long serialVersionUID = 6831768124137888591L;

		public SURFFeature(double x, double y, double[] values){
			this.x = x;
			this.y = y;
			this.values = values;
		}
	}
	
	LinkedList <SURFFeature> features = new LinkedList<SURFFeature>();
	
	public void addFeature(double x, double y, double [] values){
		features.add(new SURFFeature(x, y, values));
	}
	
	public Iterator <SURFFeature> iterator(){
		return features.iterator();
	}
	
	public int getSize(){
		return features.size();
	}
	
	public void setID(String id){
		this.ID = id;
	}
	
	public String getID(){
		return ID;
	}

	class DataInputImpl extends InputStream{
		DataInput in;
		
		public DataInputImpl(DataInput in) {
			super();
			this.in = in;
		}

		@Override
		public int read() throws IOException {
			// TODO Auto-generated method stub
			return in.readUnsignedByte();
		}
		
	}
	
	@Override
	public void readFields(DataInput in) throws IOException {
		// FIXME magic number, dont sure if this is enough
		ObjectInputStream input = new ObjectInputStream(new DataInputImpl(in));
		try {
			SURFFeatures feats = (SURFFeatures) input.readObject();
			this.setID(feats.ID);
			this.features = feats.features;
		} catch (ClassNotFoundException e) {
			throw new IOException("Could not deserialize SURFFeatures because of ClassNotFoundException", e);
		} finally {
			input.close();
		}
	}

	class DataOutputImpl extends OutputStream{
		private DataOutput out;

		public DataOutputImpl(DataOutput out){
			super();
			this.out = out;
		}
		
		@Override
		public void write(int b) throws IOException {
			out.writeByte(b);
		}
	};
	
	@Override
	public void write(DataOutput out) throws IOException {
		ObjectOutputStream objectOut = new ObjectOutputStream(new BufferedOutputStream(new DataOutputImpl(out)));
		objectOut.writeObject(this);
		objectOut.close();
	}
}
