/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.features.extraction;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Serializable;

import de.arimond.dt.CommandFailedException;
import de.arimond.dt.Util;
import de.arimond.dt.features.SURFFeatures;


/**
 * this uses the SURF library:
 * 
 * Copyright (2006): ETH Zurich, Switzerland
 * Katholieke Universiteit Leuven, Belgium
 * All rights reserved.
 * 
 * For details, see the paper:
 * Herbert Bay,  Tinne Tuytelaars,  Luc Van Gool,
 * "SURF: Speeded Up Robust Features"
 * Proceedings of the ninth European Conference on Computer Vision, May 2006
 * 
 * @author Alexander Arimond
 *
 */



public class SURFFeatureExtractor implements Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = -7601787714147751475L;
	

	private static File SURF_EXECUTABLE = new File("/home2/a_arimon/invire/InViRe/external/SURF-V1.0.9/surf.ln");
	private static double THRESHOLD = new Double(500);
	
	
	// this is not needed right now, but maybe later if feature detection and extraction step may be divided
	/**
	 * this extracts Surf features by detected regions. Not tested yet.
	 */
	/*
	
	public static SURFFeatures extractFeaturesFromDetectedRegions(File grayImage, File regionsFile, File workingDirectory) throws IOException, InterruptedException, CommandFailedException {
		final File featuresFile = new File(workingDirectory, "features.txt");
		Util.runCommand(SURF_EXECUTABLE.getAbsolutePath() + 
				" -i " + grayImage.getAbsolutePath() +
				" -o " + featuresFile.getAbsolutePath() +
				" -p1 " + regionsFile.getAbsolutePath(),
				workingDirectory);
		final BufferedReader featuresFileReader = new BufferedReader(new InputStreamReader(new FileInputStream(featuresFile)));
		
		SURFFeatures features = new SURFFeatures();
		featuresFileReader.readLine(); // first two lines
		featuresFileReader.readLine(); // skipped
		while (featuresFileReader.ready()){
			String [] values = featuresFileReader.readLine().split(" ");
			double x = Double.parseDouble(values[0]);
			double y = Double.parseDouble(values[1]);
			double [] doubleValues = new double[128];
			for (int i=5; i<values.length;i++){
				doubleValues[i-5] = Double.parseDouble(values[i]);
			}
			features.addFeature(x, y, doubleValues);
		}
		featuresFileReader.close();
		//featuresFile.delete();
		return features;
	}
*/
	
	// this is not needed right now, but maybe later if feature detection and extraction step may be divided
	/**
	 * this detects regions by using Surf detector
	 */
	/*
	
	public static File detectRegions(File grayImage, File workingDirectory) throws IOException, InterruptedException, CommandFailedException {
		final File interestPointsFile = new File(workingDirectory, grayImage.getName() + ".interestPoints.txt");
		final File regionsFile = new File(workingDirectory, "regions.txt");
		Util.runCommand(SURF_EXECUTABLE.getAbsolutePath() +
				" -i " + grayImage.getAbsolutePath() + 
				" -o " + interestPointsFile.getAbsolutePath() +
				" -thresh " + THRESHOLD, workingDirectory);
		final BufferedReader interestPointsFileReader = new BufferedReader(new InputStreamReader(new FileInputStream(interestPointsFile)));
		final BufferedWriter regionsFileWriter = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(regionsFile)));
		interestPointsFileReader.readLine(); // read the
		interestPointsFileReader.readLine(); // first to lines
		while (interestPointsFileReader.ready()){ 
			String line = interestPointsFileReader.readLine();
			String [] values = line.split(" ");
			// just take the first 5 values.. dont know why, its just taken from invire.
			String newLine = values[0] + " " + values[1] + " " + values[2] + " " + values[3] + " " + values[4];
			regionsFileWriter.write(newLine + "\n");
		}
		interestPointsFileReader.close();
		regionsFileWriter.close();
		return regionsFile;
	}

*/

	public static SURFFeatures extractFeatures(File grayImage, File workingDirectory) throws CommandFailedException, IOException, InterruptedException {
		// FIXME this features.txt is not a thread safe name
		System.out.println("extract features");
		final File featuresFile = new File(workingDirectory, grayImage.getName() + ".features.txt");
		Util.runCommand(SURF_EXECUTABLE.getAbsolutePath() +
				" -i " + grayImage.getAbsolutePath() + 
				" -o " + featuresFile.getAbsolutePath() +
				" -thresh " + THRESHOLD + 
				" -e",  // this takes 128 descriptor values 
				workingDirectory);
		final BufferedReader featuresFileReader = new BufferedReader(new InputStreamReader(new FileInputStream(featuresFile)));
		featuresFileReader.readLine();
		featuresFileReader.readLine();
		SURFFeatures features = new SURFFeatures();
		while (featuresFileReader.ready()){
			String [] values = featuresFileReader.readLine().split(" ");
			double x = Double.parseDouble(values[0]);
			double y = Double.parseDouble(values[1]);
			double [] doubleValues = new double[128];
			assert(values.length == 6 + 128);
			for (int i=6; i<values.length;i++){
				doubleValues[i-6] = Double.parseDouble(values[i]);
			}
			features.addFeature(x, y, doubleValues);
		}
		featuresFileReader.close();
		featuresFile.delete();
		return features;
	}
	
}
