/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.grid;

import org.gridgain.examples.jbosscache.GridJbossCacheManager;
import org.gridgain.grid.GridConfigurationAdapter;
import org.gridgain.grid.GridException;
import org.gridgain.grid.GridFactory;
import org.gridgain.grid.GridFactoryState;
import org.gridgain.grid.GridTask;
import org.gridgain.grid.GridTaskFuture;
import org.gridgain.grid.spi.communication.coherence.GridCoherenceCommunicationSpi;
import org.gridgain.grid.spi.discovery.coherence.GridCoherenceDiscoverySpi;
import org.gridgain.grid.spi.failover.always.GridAlwaysFailoverSpi;
import org.gridgain.grid.spi.loadbalancing.coherence.GridCoherenceLoadBalancingSpi;
import org.gridgain.grid.thread.GridThreadPoolExecutorService;

/**
 * 
 * @author Alexander Arimond
 *
 */
public class Grid {
	
	private static org.gridgain.grid.Grid grid;
	
	public static void startGridJBoss(){
		// FIXME
		try{
			String config = "config/jbosscache/spring-jbosscache.xml";
		
			GridJbossCacheManager.getInstance().start();

			GridFactory.start(config);
		} catch (GridException e){
			e.printStackTrace();
		}
	}
	
	
	public static void startGridCoherence(){
		// TODO maybe make further development here
		try {
			GridConfigurationAdapter c = new GridConfigurationAdapter();
			GridThreadPoolExecutorService g = new GridThreadPoolExecutorService();
			//GridAlwaysFailoverSpi failSpi = new GridAlwaysFailoverSpi();

			// Override maximum failover attempts.
			//failSpi.setMaximumFailoverAttempts(2);

			// Override default failover SPI.
			//c.setFailoverSpi(failSpi);
			g.setCorePoolSize(2);
			g.setMaximumPoolSize(2);
			c.setExecutorService(g);
			
			// those enables gridgain to use a in-memory data grid
			GridCoherenceCommunicationSpi comspi = new GridCoherenceCommunicationSpi();
			c.setCommunicationSpi(comspi);
			
			GridCoherenceLoadBalancingSpi clbs = new GridCoherenceLoadBalancingSpi();
			c.setLoadBalancingSpi(clbs);
			
			GridCoherenceDiscoverySpi cds = new GridCoherenceDiscoverySpi();
			c.setDiscoverySpi(cds);

			GridFactory.start(c);
			
		} catch (GridException e) {
			e.printStackTrace();
		}
	}

	public static void stopGrid(){
		GridJbossCacheManager.getInstance().stop();
		GridFactory.stop(true);
	}
	
	public static boolean isEnabled(){
		if (GridFactory.getState().equals(GridFactoryState.STARTED))
			return true;
		else
			return false;
	}
	
	public static <T,R> GridTaskFuture<R> execute(GridTask<T,R> task, T arg) throws Exception{
		if (isEnabled()){
			return GridFactory.getGrid().execute(task, arg);
		} else {
			// FIXME dont like this one
			throw new Exception("Grid has not been started.");
		}
	}

}
