/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution;

import java.io.Serializable;

import com.rapidminer.operator.IOObject;

import de.arimond.dt.operators.distribution.gridgain.GridGainCoherenceDistribution;
import de.arimond.dt.operators.distribution.hadoop.HadoopDistribution;


/**
 * 
 * @author Alexander Arimond
 *
 */
public class DistributionFactory{

	public enum DistributionType{
		GRIDGAIN,
		HADOOP
	}
	
	public enum DistributionDataLayerType{
		COHERENCE,
		JBOSSCACHE,
	}
	
	public static <I extends IOObject, S extends Serializable, M extends Serializable, O extends Serializable> Distribution<S,M,O> create (DistributionType type, DistributionDataLayerType dataLayerType, DistributionMapReduce <S,M,O> spec){
		if (spec == null){
			throw new NullPointerException("MapReduceSpecification must not be null!");
		}
	
		switch(type){
			case GRIDGAIN:
				switch(dataLayerType){
					case COHERENCE:
						return new GridGainCoherenceDistribution<S, M, O>(spec);
				}
			case HADOOP:
				return new HadoopDistribution<S,M,O>(spec);
		}
		throw new IllegalArgumentException("No valid type for distribution: \"" + type+ "\"");
	}
	
}
