/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution;

import java.io.Serializable;
import java.util.List;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <S>
 * @param <M>
 * @param <O>
 */

public abstract class DistributionMapReduce <S extends Serializable, M extends Serializable, O extends Serializable> implements Serializable{
	

	/*
	 * This splits the Input of the whole map-reduce process. The splits are forwarded to different nodes which
	 * perform the map function on it.
	 * @param input
	 * @return List of splits.
	 * @throws Exception
	 */
	// public abstract List <S> split (I input) throws Exception;

//	public void setDataLayer(DistributionDataLayer dataLayer){
//		this.dataLayer = dataLayer;
//	}
	
	// private DistributionDataLayer dataLayer;
	
	/**
	 * 
	 */
	private static final long serialVersionUID = 4066485652630942493L;

	/**
	 * This function will be performed on each split in a distributed way.
	 * The output will be forwarded to exactly one reduce function.
	 * @param split
	 * @return
	 * @throws Exception
	 */
	public abstract M map (DistributionDataLayer <S> dataLayer, int mapNr) throws Exception;
	
	/**
	 * This function collects all map outputs and reduces them to a single output.
	 * @param mapResults
	 * @return
	 * @throws Exception
	 */
	
	public abstract O reduce(List<M> mapResults, DistributionDataLayer <S> dataLayer) throws Exception;
	
}
