/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.clustering2;

import java.util.List;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Attributes;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.clustering.CentroidClusterModel;
import com.rapidminer.tools.math.similarity.DistanceMeasure;

import de.arimond.dt.operators.distribution.DistributionDataLayer;
import de.arimond.dt.operators.distribution.DistributionMapReduce;

public class FastKMeansInitSpec extends
		DistributionMapReduce<ExampleSet, Assignments, Boolean> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -6024848966468653682L;

	private double[] getAsDoubleArray(Example example, Attributes attributes) {
		double[] values = new double[attributes.size()];
		int i = 0;
		for (Attribute attribute : attributes) {
			values[i] = example.getValue(attribute);
			i++;
		}
		return values;
	}
	
	@Override
	public Assignments map(DistributionDataLayer<ExampleSet> dataLayer,
			int mapNr) throws Exception {
		
		ExampleSet exampleSet = (ExampleSet) dataLayer.getMapInputData(mapNr);
		SplitInformation splitInfo = (SplitInformation) dataLayer.getMapData(mapNr, "split_data");
		
		if (exampleSet == null) throw new NullPointerException("exampleSet must not be null");
		if (splitInfo == null) throw new NullPointerException("split info must not be null");

		DistanceMeasure measure = (DistanceMeasure)dataLayer.getData("distanceMeasure");
		CentroidClusterModel model = (CentroidClusterModel) dataLayer.getData("model");
		ClusterDistances cDistances = (ClusterDistances) dataLayer.getData("clusterDistances");
		Attributes attributes = (Attributes) dataLayer.getData("attributes");
		
		splitInfo.initializeFields();
		double [][] l = splitInfo.l;
		double [] u = splitInfo.u;
		boolean [] r = splitInfo.r;
		int [] centroidAssignments = splitInfo.c;
		
		int x = 0;
		for (Example example : exampleSet) {
			double[] exampleValues = getAsDoubleArray(example, attributes);
			double nearestDistance = measure.calculateDistance(model.getCentroidCoordinates(0), exampleValues);
			l[x][0] = nearestDistance;
			int nearestIndex = 0;
			for (int centroidIndex = 1; centroidIndex < model.getNumberOfClusters(); centroidIndex++) {
				if (cDistances.matrix.get(nearestIndex, centroidIndex) >= 2 * nearestDistance) continue;
				final double distance = measure.calculateDistance(model.getCentroidCoordinates(centroidIndex), exampleValues);
				l[x][centroidIndex] = distance;
				if (distance < nearestDistance) {
					nearestDistance = distance;
					nearestIndex = centroidIndex;
				}
			}
			centroidAssignments[x] = nearestIndex;
			u[x] = nearestDistance;
			r[x] = false;
			x++;
		}
		dataLayer.putMapData(mapNr, "split_data", splitInfo);
		// just empty
		return new Assignments(0, null, null);
	}

	@Override
	public Boolean reduce(List<Assignments> mapResults,
			DistributionDataLayer<ExampleSet> dataLayer) throws Exception {
		return new Boolean(true);
	}

}
