/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.clustering3;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.AttributeRole;
import com.rapidminer.example.Attributes;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.Tools;
import com.rapidminer.example.set.SplittedExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.UserError;
import com.rapidminer.operator.clustering.CentroidClusterModel;
import com.rapidminer.operator.clustering.ClusterModel;
import com.rapidminer.operator.clustering.clusterer.RMAbstractClusterer;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeCategory;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.parameter.ParameterTypeString;
import com.rapidminer.parameter.UndefinedParameterError;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.RandomGenerator;
import com.rapidminer.tools.math.similarity.DistanceMeasure;
import com.rapidminer.tools.math.similarity.numerical.EuclideanDistance;

import de.arimond.dt.TimeMeasurement;
import de.arimond.dt.operators.distribution.Distribution;
import de.arimond.dt.operators.distribution.DistributionException;
import de.arimond.dt.operators.distribution.DistributionFactory;
import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.distribution.DistributionFactory.DistributionDataLayerType;
import de.arimond.dt.operators.distribution.DistributionFactory.DistributionType;
import de.arimond.dt.operators.distribution.gridgain.Grid;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class DistributedKMeans extends com.rapidminer.operator.clustering.clusterer.KMeans{

	private static final String PARAMETER_SPLITS = "splits";

	public DistributedKMeans(OperatorDescription description) {
		super(description);
	}

	private Distribution<ExampleSet, KMeansResult, KMeansResult> distribution;

	private double[] getAsDoubleArray(Example example, Attributes attributes) {
		double[] values = new double[attributes.size()];
		int i = 0;
		for (Attribute attribute : attributes) {
			values[i] = example.getValue(attribute);
			i++;
		}
		return values;
	}
	
	// make a new ExampleSet which only contains the data of current subset
	public static ExampleSet cloneSubset(SplittedExampleSet set, int nr) {
		
		set.selectSingleSubset(nr);
		
//		// old table
//		ExampleTable oldTable = set.getExampleTable();
//		
//		// old attributes
//		Iterator <AttributeRole> all = set.getAttributes().allAttributeRoles();
//		
//		// copied attributes and specials mapping
//		LinkedList <Attribute> allAttrs = new LinkedList<Attribute>();
//		HashMap <Attribute, String> specialsMapping = new HashMap <Attribute, String>(); 
//		while (all.hasNext()){
//			AttributeRole ar = all.next();
//			Attribute a = ar.getAttribute();
//			Attribute copyOfA = AttributeFactory.createAttribute(a.getName(), a.getValueType());
//			copyOfA.setBlockType(a.getBlockType());
//			if (a.isNominal())
//				copyOfA.setMapping(a.getMapping());
//			copyOfA.setConstruction(a.getConstruction());
//			copyOfA.setDefault(a.getDefault());
//			copyOfA.setTableIndex(a.getTableIndex());
//			if (ar.isSpecial()){
//				specialsMapping.put(copyOfA, ar.getSpecialName());
//			}
//			allAttrs.add(copyOfA);
//		}
//
//		// new example table with copied attributes
//		MemoryExampleTable table = new MemoryExampleTable(allAttrs);
//
//		// copy all data rows
//		for (int i  = 0; i < set.size(); i++){
//			DataRow dataRow = set.getExample(i).getDataRow();
//            double [] newDataRowData = new double[oldTable.getNumberOfAttributes()];
//			for (int a = 0; a < oldTable.getNumberOfAttributes(); a++) {
//                Attribute attribute = oldTable.getAttribute(a);
//                if (attribute != null) {
//                    newDataRowData[a] = dataRow.get(attribute);
//                } else {
//                    newDataRowData[a] = Double.NaN;
//                }
//            }
//			table.addDataRow(new DoubleArrayDataRow(newDataRowData));
//		}
//		
//		// create example set and return
//		final ExampleSet resultSet = table.createExampleSet(specialsMapping);
//		return resultSet;
//
		Iterator <AttributeRole> all = set.getAttributes().allAttributeRoles();
		
		// FIXME this method does not "copy" the examples, the corresponding data rows are still linked to the old attributes
		// example.getValue(attr) therefore will only work on the original attributes
		
		// copied attributes and specials mapping
		LinkedList <Attribute> allAttrs = new LinkedList<Attribute>();
		HashMap <Attribute, String> specialsMapping = new HashMap <Attribute, String>(); 
		while (all.hasNext()){
			AttributeRole ar = all.next();
			Attribute a = ar.getAttribute();
			Attribute copyOfA = AttributeFactory.createAttribute(a.getName(), a.getValueType());
			copyOfA.setBlockType(a.getBlockType());
			if (a.isNominal())
				copyOfA.setMapping(a.getMapping());
			copyOfA.setConstruction(a.getConstruction());
			copyOfA.setDefault(a.getDefault());
			copyOfA.setTableIndex(a.getTableIndex());
			if (ar.isSpecial()){
				specialsMapping.put(copyOfA, ar.getSpecialName());
			}
			allAttrs.add(copyOfA);
		}
	
		MemoryExampleTable table = new MemoryExampleTable(allAttrs);
		for (Example example : set){
			table.addDataRow(example.getDataRow());
		}
		return table.createExampleSet(specialsMapping);
	}
	
	
	@Override
	public ClusterModel generateClusterModel(ExampleSet exampleSet) throws OperatorException {	
		int k = getParameterAsInt(PARAMETER_K);
		int maxOptimizationSteps = getParameterAsInt(PARAMETER_MAX_OPTIMIZATION_STEPS);
		int maxRuns = getParameterAsInt(PARAMETER_MAX_RUNS);
		DistanceMeasure measure = new EuclideanDistance();
		measure.init(exampleSet);

		// checking and creating ids if necessary
		Tools.checkAndCreateIds(exampleSet);

		// additional checks
		Tools.onlyNonMissingValues(exampleSet, "KMeans");
		if (exampleSet.size() < k) {
			throw new UserError(this, 142, k);
		}

		// extracting attribute names
		Attributes attributes = exampleSet.getAttributes();
		ArrayList<String> attributeNames = new ArrayList<String>(attributes.size());
		for (Attribute attribute : attributes)
			attributeNames.add(attribute.getName());

		RandomGenerator generator = RandomGenerator.getRandomGenerator(this);
		double minimalIntraClusterDistance = Double.POSITIVE_INFINITY;
		CentroidClusterModel bestModel = null;
		int[] bestAssignments = null;


		//System.getProperties().setProperty("tangosol.coherence.distributed.strict", "false");
		
		TimeMeasurement time = new TimeMeasurement();
		time.start();
		
		DistributionType type = getDistributionType();
		DistributionDataLayerType layerType = getDistributionDataLayerType();
		
		if (type.equals(DistributionType.GRIDGAIN) && layerType.equals(DistributionDataLayerType.COHERENCE)) Grid.startGridCoherence();

		DistributionMapReduce <ExampleSet, KMeansResult, KMeansResult> spec = getDistributionMapReduce();
		distribution = DistributionFactory.create(type, layerType, spec);
		distribution.setNumberMaps(maxRuns);
	
		for (int i = 0; i < maxRuns; i++){
			distribution.provideMapInputData(i, exampleSet);
			distribution.provideMapData(i, "seed", new Long(generator.nextLong()));
		}
		
		distribution.provideStaticData("k", new Integer(k));
		distribution.provideStaticData("distanceMeasure", measure);
		distribution.provideStaticData("attributes", attributes);
		// distribution.provideStaticData("generator", generator);
		distribution.provideStaticData("attributes", attributes);
		distribution.provideStaticData("attributeNames", attributeNames);
		
		distribution.provideStaticData("maxOptimizationSteps", new Integer(maxOptimizationSteps));
		
		boolean add_as_label = getParameterAsBoolean(RMAbstractClusterer.PARAMETER_ADD_AS_LABEL);
		distribution.provideStaticData("add_as_label", new Boolean(add_as_label));
		boolean remove_unlabeled = getParameterAsBoolean(RMAbstractClusterer.PARAMETER_REMOVE_UNLABELED);
		distribution.provideStaticData("remove_unlabeled", remove_unlabeled);
		
		TimeMeasurement t = new TimeMeasurement();
		t.start();
		
		KMeansResult result;
		try {
			result = distribution.startComputation();
		} catch (DistributionException e) {
			throw new OperatorException("Distribution failed for some reason.", e);
		} finally {
			distribution.clearData();
			if (type.equals(DistributionType.GRIDGAIN))
				Grid.stopGrid();
		}
		t.print("time ");

		bestModel = result.getModel();
		bestAssignments = result.getAssignments();

		bestModel.setClusterAssignments(bestAssignments, exampleSet);

		if (addsClusterAttribute()) {
			Attribute cluster = AttributeFactory.createAttribute("cluster", Ontology.NOMINAL);
			exampleSet.getExampleTable().addAttribute(cluster);
			exampleSet.getAttributes().setCluster(cluster);
			int i = 0;
			for (Example example : exampleSet) {
				example.setValue(cluster, "cluster_" + bestAssignments[i]);
				i++;
			}
		}
		
		String message = getParameterAsString("messageForTime");
		t.appendToFile(new File("/home2/a_arimon/evaluation/DKMeans_iter.txt"), message);
		return bestModel;
	}
	
	
	private DistributionMapReduce <ExampleSet, KMeansResult, KMeansResult> getDistributionMapReduce() {
		return new KMeansMapReduceSpec();
	}

	private DistributionDataLayerType getDistributionDataLayerType() throws UndefinedParameterError {
		return DistributionDataLayerType.valueOf(getParameterAsString(PARAMETER_DATALAYER));
		//return DistributionDataLayerType.COHERENCE;
	}

	private DistributionType getDistributionType() throws UndefinedParameterError {
		return DistributionType.valueOf(getParameterAsString(PARAMETER_MODE));
		//return DistributionType.GRIDGAIN;
	}

//	private void printArray(int [] a, String name){
//		System.out.print(name + " ");
//		for (int i =0 ; i< a.length; i++){
//			System.out.print(a[i] + " ");
//		}
//	}
//	
//	private void printArray(double [] a, String name){
//		System.out.print(name + " ");
//		for (int i =0 ; i< a.length; i++){
//			System.out.print(a[i] + " ");
//		}
//	}
	
//	private int [] computeAssignments(CentroidClusterModel model, int k) throws GridTaskTimeoutException, GridException, UndefinedParameterError {
//		task.setSlits(getParameterAsInt(PARAMETER_SPLITS));
//		final GridTaskFuture <Assignments> future = GridFactory.getGrid().execute(task, model);
//		final Assignments assigns = future.get();
//		for (int j = 0; j< k ; j++){
//			model.getCentroid(j).assignManyExamples(assigns.getCentroidSums()[j], assigns.getNrOfAssigns()[j]);
//		}
//		return assigns.getCentroidAssignments();
//	}

	private final String PARAMETER_MODE = "mode";
	private final String PARAMETER_DATALAYER = "datalayer";
	private final String STANDALONE_MODE = "STANDALONE";
	
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List<ParameterType> types = super.getParameterTypes();
		types.add(new ParameterTypeInt(PARAMETER_SPLITS, "splits", 2, 100, 20));
		ParameterType type = new ParameterTypeCategory(PARAMETER_MODE,
				"standalone, gridgain or hadoop", 
				new String [] {STANDALONE_MODE, DistributionType.GRIDGAIN.toString(), DistributionType.HADOOP.toString()}, 
				0);
		types.add(type);
		ParameterType type1 = new ParameterTypeCategory(PARAMETER_DATALAYER,
				"which data layer to use when distributing with gridgain", 
				new String [] {DistributionDataLayerType.COHERENCE.toString()}, 
				0);
		types.add(type1);
		// FIXME delete this later
		ParameterTypeString type2 = new ParameterTypeString("messageForTime", "");
		types.add(type2);
		return types;
	}
	
}
