/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.coherence;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;

import com.tangosol.coherence.component.util.daemon.queueProcessor.service.grid.DistributedCache;
import com.tangosol.net.CacheFactory;
import com.tangosol.net.DefaultConfigurableCacheFactory;
import com.tangosol.net.NamedCache;
import com.tangosol.net.PartitionedService;
import com.tangosol.run.xml.XmlElement;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class CacheSingleton {

	private NamedCache cache;
	private NamedCache cache_static;
	private ClassLoader currentClassLoader = null;
	
	private static CacheSingleton instance;

	private CacheSingleton(ClassLoader cl){
		try{
			if (cache != null 
					&& cache.isActive() 
					&& cache.getCacheService().isRunning()
					&& cache_static != null
					&& cache_static.isActive()
					&& cache_static.getCacheService().isRunning()
					&& cl.equals(currentClassLoader)) {
				return;
			} else {
				
				if (cache != null) cache.release();
				if (cache_static != null) cache_static.release();
				if (currentClassLoader == null || !cl.equals(currentClassLoader))currentClassLoader = cl;
				
				DefaultConfigurableCacheFactory factory = (DefaultConfigurableCacheFactory)CacheFactory.getConfigurableCacheFactory(currentClassLoader);

				// this does not work for some reason
				//final XmlElement config =  DefaultConfigurableCacheFactory.loadConfigAsResource("de/arimond/dt/operators/distribution1/coherence/config.xml", currentClassLoader);
				final XmlElement config = getConfig();

				factory.setConfig(config);

				cache = CacheFactory.getCache(CoherenceDataLayer.CACHE_NAME);
				cache_static = CacheFactory.getCache(CoherenceDataLayer.STATIC_CACHE_NAME);
			}
		} catch (IOException ioe){
			//FIXME
			System.out.println("FAILURE FAILURE FAILURE");
		}
	}

	
	// this loads the configuration file for the cache, either from local classloader or GridDeploymentClassLoader, stores the configuration in
	// temporary file and reads it due to DefaultConfigurableCacheFactory.loadConfigFromFile()
	// DefaultConfigurableCacheFactory.loadConfigAsResource does not work for some reason
	private XmlElement getConfig() throws IOException {
		File temp = null; 
		try{
			temp = File.createTempFile(CoherenceDataLayer.CACHE_NAME + "_config", java.util.UUID.randomUUID().toString());
			FileOutputStream out = new FileOutputStream(temp);
			// FIXME resource address
			//InputStream input = this.getClass().getClassLoader().getResourceAsStream("de/arimond/dt/operators/distribution1/coherence/config.xml");
			InputStream input = currentClassLoader.getResourceAsStream("de/arimond/dt/operators/distribution1/coherence/config.xml");
			byte [] bbuf = new byte[512];
			while (true){
				int bytes = input.read(bbuf);
				if (bytes <= 0) break;
				out.write(bbuf);
			}
			out.close(); 
			input.close();

			return DefaultConfigurableCacheFactory.loadConfigFromFile(temp);
		}
		catch (IOException ioe){
			throw ioe;
		}finally{
			if (temp != null)
			temp.delete();
		}
	}

	public NamedCache getCache(){
		return cache;
	}
	
	public NamedCache getStaticCache(){
		return cache;
	}

	public static synchronized CacheSingleton getInstance(ClassLoader cl){
		if (instance == null){
			System.out.println("creating new cache instance");
			instance = new CacheSingleton(cl);
		}
		return instance;
	}
	
	public void shutdown(){
		cache.getCacheService().shutdown();
		cache_static.getCacheService().shutdown();
	}
	
}
