/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.coherence;

import java.io.Serializable;
import java.util.UUID;

import org.gridgain.grid.GridFactory;
import org.gridgain.grid.spi.discovery.coherence.GridCoherenceDiscoverySpi;
import org.gridgain.grid.spi.discovery.coherence.GridCoherenceMember;

import com.tangosol.net.Member;
import com.tangosol.net.NamedCache;
import com.tangosol.net.PartitionedService;
import com.tangosol.util.InvocableMap.EntryProcessor;

import de.arimond.dt.operators.distribution.DistributionDataLayer;

/**
 * 
 * author Alexander Arimond
 *
 * @param <S>
 */

public class CoherenceDataLayer <S extends Serializable> extends DistributionDataLayer <S>{

	public static String CACHE_NAME = "distribution";
	public static String STATIC_CACHE_NAME = "static";
	
	public static NamedCache cache;
	private UUID distributionUUID;
	
	public boolean debug = true;
	private ClassLoader cl;
	
	public CoherenceDataLayer(UUID distributionUUID, ClassLoader cl){
		this.distributionUUID = distributionUUID;
		this.cl = cl;
	}
	
	@Override
	public Serializable getData(Serializable key) {
		return (Serializable) CacheSingleton.getInstance(cl).getStaticCache().get(key);
	}

	@Override
	public Serializable getMapData(int mapNr, Serializable key) {
		MapDataKey mapDataKey = new MapDataKey(distributionUUID, mapNr, key); 

		if (debug){
			
			PartitionedService service = (PartitionedService) CacheSingleton.getInstance(cl).getCache().getCacheService();
			Member mapDataMember  = service.getKeyOwner(mapDataKey);
			Member mapInputDataMember = service.getKeyOwner(new MapDataKey(distributionUUID, mapNr));
			boolean colocated = false;
			if (mapDataMember!= null && mapInputDataMember != null && mapDataMember.getId() == mapInputDataMember.getId()){
				colocated = true;
			}
//			PartitionedService service = (PartitionedService) CacheSingleton.getInstance().getCache().getCacheService();
//			Member mapInputDataMember = service.getKeyOwner(new MapDataKey(distributionUUID, mapNr, key));
//			GridCoherenceMember gridMem = (GridCoherenceMember)GridFactory.getGrid().getLocalNode().getAttribute(
//					GridCoherenceDiscoverySpi.ATTR_COHERENCE_MBR);
//			boolean colocated = false;
//			if (mapInputDataMember != null && gridMem != null && mapInputDataMember.getId() == gridMem.getId()){
//				colocated = true;
//			}
			System.out.println("DATA FOR MAP " + mapNr + " COLOCATED: " + colocated);
		}
		if (!CacheSingleton.getInstance(cl).getCache().containsKey(mapDataKey)){
			throw new NullPointerException("key not hold in cache " + new MapDataKey(distributionUUID, mapNr, key));
		}
		
		CacheSingleton.getInstance(cl).getStaticCache().lock(mapDataKey);
		Serializable data = null;
		try{
			data = (Serializable) CacheSingleton.getInstance(cl).getCache().get(mapDataKey);
		} finally{
			CacheSingleton.getInstance(cl).getStaticCache().unlock(mapDataKey);
		}
		if (data == null){
			throw new NullPointerException("did not found data for key " + new MapDataKey(distributionUUID, mapNr, key));
		}
		return data;
	}

	@Override
	public S getMapInputData(int mapNr) {
		
		MapDataKey mKey = new MapDataKey(distributionUUID, mapNr);
		if (debug){
			PartitionedService service = (PartitionedService) CacheSingleton.getInstance(cl).getCache().getCacheService();
			Member mapInputDataMember = service.getKeyOwner(mKey);
			GridCoherenceMember gridMem = (GridCoherenceMember)GridFactory.getGrid().getLocalNode().getAttribute(
					GridCoherenceDiscoverySpi.ATTR_COHERENCE_MBR);
			boolean affinity = false;
			if (mapInputDataMember != null && gridMem != null && mapInputDataMember.getId() == gridMem.getId()){
				affinity = true;
			}
			System.out.println("DATA FOR MAP " + mapNr + " AFFINITY: "+ affinity);

		}
		
		CacheSingleton.getInstance(cl).getStaticCache().lock(mKey);
		try{
		return (S) CacheSingleton.getInstance(cl).getCache().get(new MapDataKey(distributionUUID, mapNr));
		} finally{
			CacheSingleton.getInstance(cl).getStaticCache().unlock(mKey);
		}
	}

	@Override
	public void putData(Serializable key, Serializable data) {
		CacheSingleton.getInstance(cl).getStaticCache().lock(key);
		try{
			CacheSingleton.getInstance(cl).getStaticCache().put(key, data);
		} finally{
			CacheSingleton.getInstance(cl).getStaticCache().unlock(key);
		}
	}

	@Override
	public void putMapData(int mapNr, Serializable key, Serializable data) {
		MapDataKey mKey = new MapDataKey(distributionUUID, mapNr, key);
		System.out.println("provided map data: key: " + mKey);
		CacheSingleton.getInstance(cl).getCache().lock(mKey);
		try{
			CacheSingleton.getInstance(cl).getCache().put(mKey, data);
		} finally {
			CacheSingleton.getInstance(cl).getCache().unlock(mKey);
		}
			
	}

	@Override
	public void putMapInputData(int mapNr, S input) {
		CacheSingleton.getInstance(cl).getCache().put(new MapDataKey(distributionUUID, mapNr), input);
	}
	
	public void clearData(){
		CacheSingleton.getInstance(cl).getCache().clear();
		CacheSingleton.getInstance(cl).getStaticCache().clear();
	}
	
	public Object invokeOnMap(int mapNr, Serializable key, EntryProcessor entryProcessor){
		MapDataKey mapDataKey = new MapDataKey(distributionUUID, mapNr, key); 
		return CacheSingleton.getInstance(cl).getCache().invoke(mapDataKey, entryProcessor);
	}
	
}
