
/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.gridgain;

import java.io.Serializable;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.gridgain.grid.GridException;
import org.gridgain.grid.GridFactory;
import org.gridgain.grid.GridJob;
import org.gridgain.grid.GridJobAdapter;
import org.gridgain.grid.GridJobResult;
import org.gridgain.grid.GridNode;
import org.gridgain.grid.GridTaskAdapter;
import org.gridgain.grid.GridTaskSession;
import org.gridgain.grid.logger.GridLogger;
import org.gridgain.grid.resources.GridLoggerResource;
import org.gridgain.grid.resources.GridTaskSessionResource;

import de.arimond.dt.operators.distribution.coherence.CacheSingleton;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class CacheManagementTask extends GridTaskAdapter<CacheManagementTask.CacheManagementTaskType, String> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -7439965673704643491L;

	public enum CacheManagementTaskType{
		INIT,
		SHUTDOWN
	}
	
	public class CacheJob extends GridJobAdapter<CacheManagementTaskType>{

		@GridLoggerResource
		GridLogger logger;
		
		@GridTaskSessionResource
		GridTaskSession gtsession;
		
		/**
		 * 
		 */
		private static final long serialVersionUID = -5068861835570594135L;

		@Override
		public Serializable execute() throws GridException {
			
			CacheManagementTaskType type = getArgument();
			
			if (type.equals(CacheManagementTaskType.INIT) && logger.isInfoEnabled())
				logger.info("Initialize Coherence Cache for Distribution");
			
			// this initializes the cache if not initialized before
			CacheSingleton instance = CacheSingleton.getInstance(gtsession.getClassLoader());
			
			if (type.equals(CacheManagementTaskType.SHUTDOWN)){
				if (logger.isInfoEnabled())
					logger.info("Shutdown Coherence Cache for Distribution");
				instance.shutdown();
			}

			return "";
		}
		
	}
	
	
	@Override
	public Map<? extends GridJob, GridNode> map(List<GridNode> subgrid,
			CacheManagementTaskType arg) throws GridException {
		HashMap <CacheJob, GridNode> jobMapping = new HashMap<CacheJob, GridNode> ();
		
		for (GridNode node : subgrid){
			if (GridFactory.getGrid().getRemoteNodes().contains(node)){
				CacheJob job = new CacheJob();
				job.setArgument(arg);
				jobMapping.put(job, node);
			}
		}
		return jobMapping;
	}

	@Override
	public String reduce(List<GridJobResult> results) throws GridException {
		return "";
	}

}
