/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.gridgain;

import java.io.Serializable;
import java.util.Collection;
import java.util.LinkedList;
import java.util.List;
import java.util.UUID;

import org.gridgain.grid.GridException;
import org.gridgain.grid.GridJob;
import org.gridgain.grid.GridJobResult;
import org.gridgain.grid.GridJobResultPolicy;
import org.gridgain.grid.GridTaskNoResultCache;
import org.gridgain.grid.GridTaskSession;
import org.gridgain.grid.GridTaskSplitAdapter;
import org.gridgain.grid.logger.GridLogger;
import org.gridgain.grid.resources.GridLoggerResource;
import org.gridgain.grid.resources.GridTaskSessionResource;
import org.gridgain.grid.spi.loadbalancing.affinity.GridAffinityJobAdapter;
import org.gridgain.grid.spi.loadbalancing.coherence.GridCoherenceAffinityJobAdapter;

import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.distribution.coherence.CoherenceDataLayer;
import de.arimond.dt.operators.distribution.coherence.MapDataKey;


/**
 * This class integrates the map-reduce functionality of gridgain into rapidminer in a generic way.
 * @author Alexander Arimond
 *
 * @param <T>
 * @param <S>
 * @param <M>
 * @param <R>
 */

public class CoherenceDistributionTask <S extends Serializable, M extends Serializable, R extends Serializable> extends GridTaskSplitAdapter <Integer, R>{

	/**
	 * 
	 */
	private static final long serialVersionUID = -6291367435063367720L;
	
	private final DistributionMapReduce <S, M, R> spec;

	private UUID distributionUUID;

	@GridLoggerResource
	private GridLogger logger;
	
	@GridTaskSessionResource
	private GridTaskSession gtsession;
	
	public CoherenceDistributionTask (DistributionMapReduce<S, M, R> mapReduceSpec, UUID distributionUUID) {
		this.spec = mapReduceSpec;
		this.distributionUUID = distributionUUID;
	}

	@Override
	protected Collection<? extends GridJob> split(int gridSize, Integer numberMaps)
			throws GridException {
		try {
			final LinkedList<GridJob> jobs = new LinkedList<GridJob>();
			
			for (int i=0; i < numberMaps; i ++){
				final MapJob <S,M,R> job = new MapJob<S,M,R>(i, spec, distributionUUID);
				jobs.add(job);
			}
			return jobs;
		} catch (Exception e){
			throw new GridException(e);
		}
	}

	@Override
	public GridJobResultPolicy result(GridJobResult result,
			List<GridJobResult> received) throws GridException {
		GridJobResultPolicy policy =  super.result(result, received);
//		if (policy.equals(GridJobResultPolicy.WAIT)){
//			resultList.add((M)result.getData());
//		}
		return policy;
	}
	
	//LinkedList<M> resultList = new LinkedList<M>();
	
	@Override
	public R reduce(List<GridJobResult> results) throws GridException {
		try {
			final LinkedList <M> mapResults =  new LinkedList<M> ();
			for (GridJobResult result : results){
				mapResults.add((M)result.getData());
			}
			//final LinkedList <M> mapResults = resultList;
			
			if (spec instanceof GridGainMapReduce<?,?,?>){
				((GridGainMapReduce<S, M, R>) spec).setGridLogger(logger);
			}
			return spec.reduce(mapResults, new CoherenceDataLayer <S> (distributionUUID, gtsession.getClassLoader()));
		} catch (Exception e){
			throw new GridException(e);
		}
	}
	

}
