/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.gridgain;

import org.gridgain.grid.GridConfigurationAdapter;
import org.gridgain.grid.GridException;
import org.gridgain.grid.GridFactory;
import org.gridgain.grid.GridFactoryState;
import org.gridgain.grid.GridTask;
import org.gridgain.grid.GridTaskFuture;
import org.gridgain.grid.GridTaskTimeoutException;
import org.gridgain.grid.spi.collision.fifoqueue.GridFifoQueueCollisionSpi;
import org.gridgain.grid.spi.communication.tcp.GridTcpCommunicationSpi;
import org.gridgain.grid.spi.discovery.coherence.GridCoherenceDiscoverySpi;
import org.gridgain.grid.spi.failover.always.GridAlwaysFailoverSpi;
import org.gridgain.grid.spi.loadbalancing.coherence.GridCoherenceLoadBalancingSpi;
import org.gridgain.grid.thread.GridThreadPoolExecutorService;

import de.arimond.dt.operators.distribution.gridgain.CacheManagementTask.CacheManagementTaskType;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class Grid {
	
	private static final int Serializable = 0;
	private static org.gridgain.grid.Grid grid;
	
//	public static void startGridJBoss(){
//		// FIXME
//		try{
//			String config = "config/jbosscache/spring-jbosscache.xml";
//		
//			GridJbossCacheManager.getInstance().start();
//
//			GridFactory.start(config);
//		} catch (GridException e){
//			e.printStackTrace();
//		}
//	}
	
	public static int nr_threads = 1;
	
	public static void startGridCoherence(){
		// TODO maybe make further development here
		try {
			GridConfigurationAdapter c = new GridConfigurationAdapter();
			
			//c.setGridLogger(new GridLog4jLogger(new File("/home2/a_arimon/gridgain/log4j.xml")));
			
			//c.setMarshaller(new GridXstreamMarshaller());
			
			GridThreadPoolExecutorService g = new GridThreadPoolExecutorService();
			//GridAlwaysFailoverSpi failSpi = new GridAlwaysFailoverSpi();

			// Override maximum failover attempts.
			//failSpi.setMaximumFailoverAttempts(2);

			// Override default failover SPI.
			//c.setFailoverSpi(failSpi);
			//g.setCorePoolSize();
			//g.setMaximumPoolSize(10);
			c.setExecutorService(g);
			
			
			//GridJobStealingCollisionSpi collisionSPI = new GridJobStealingCollisionSpi();
			//collisionSPI.setActiveJobsThreshold(nr_threads);
			GridFifoQueueCollisionSpi collisionSPI = new GridFifoQueueCollisionSpi();
			collisionSPI.setParallelJobsNumber(nr_threads);

			c.setCollisionSpi(collisionSPI);
			
			// those enables gridgain to use a in-memory data grid
//			GridCoherenceCommunicationSpi comspi = new GridCoherenceCommunicationSpi();
//			c.setCommunicationSpi(comspi);
			
			GridTcpCommunicationSpi commSpi = new GridTcpCommunicationSpi();
			commSpi.setIdleConnectionTimeout(50000);
			commSpi.setMessageThreads(10);
			commSpi.setLocalPort(47300);
			commSpi.setDirectBuffer(false);
			c.setCommunicationSpi(commSpi);
			
			GridCoherenceLoadBalancingSpi clbs = new GridCoherenceLoadBalancingSpi();
			//GridAffinityLoadBalancingSpi clbs = new GridAffinityLoadBalancingSpi();
			c.setLoadBalancingSpi(clbs);
			
			GridCoherenceDiscoverySpi cds = new GridCoherenceDiscoverySpi();
			c.setDiscoverySpi(cds);

			c.setPeerClassLoadingTimeout(10000);
			
			
			GridAlwaysFailoverSpi failoverSpi = new GridAlwaysFailoverSpi();
			failoverSpi.setMaximumFailoverAttempts(0);
			c.setFailoverSpi(failoverSpi);
			
			grid = GridFactory.start(c);
			
			if (!grid.getRemoteNodes().isEmpty()){
				GridTaskFuture <String> future = grid.execute(new CacheManagementTask(), CacheManagementTaskType.INIT);
				future.get();
			}
			
		} catch (GridException e) {
			e.printStackTrace();
		}
	}

	public static void stopGrid(){
//		GridJbossCacheManager.getInstance().stop();
		GridTaskFuture <String> future = grid.execute(new CacheManagementTask(), CacheManagementTaskType.SHUTDOWN);
		try {
			future.get();
		} catch (GridTaskTimeoutException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (GridException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

//		CacheSingleton single = CacheSingleton.getInstance(Grid.class.getClassLoader());
//		single.destroy();
//		single.getCache().getCacheService().shutdown();

		GridFactory.stop(true);
	}
	
	public static boolean isEnabled(){
		if (GridFactory.getState().equals(GridFactoryState.STARTED))
			return true;
		else
			return false;
	}
	
	public static <T,R> GridTaskFuture<R> execute(GridTask<T,R> task, T arg) throws Exception{
		if (isEnabled()){
			return GridFactory.getGrid().execute(task, arg);
		} else {
			// FIXME dont like this one
			throw new Exception("Grid has not been started.");
		}
	}

}
