/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.gridgain;

import java.io.Serializable;
import java.util.UUID;

import org.gridgain.grid.GridException;
import org.gridgain.grid.GridTaskFuture;
import org.gridgain.grid.GridTaskTimeoutException;

import de.arimond.dt.operators.distribution.Distribution;
import de.arimond.dt.operators.distribution.DistributionDataLayer;
import de.arimond.dt.operators.distribution.DistributionException;
import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.distribution.coherence.CoherenceDataLayer;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <S>
 * @param <M>
 * @param <O>
 */

public class GridGainCoherenceDistribution <S extends Serializable,M extends Serializable,O extends Serializable> extends Distribution <S,M,O>{

	public GridGainCoherenceDistribution(DistributionMapReduce<S, M, O> spec) {
		super(spec);
	}

	private UUID distributionUUID = UUID.randomUUID();
	
	@Override
	public O startComputation() throws DistributionException {
			try {
				final GridTaskFuture <O> future = Grid.execute(new CoherenceDistributionTask <S, M, O >(this.getMapReduceSpecification(), distributionUUID), new Integer(numberMaps));
				return future.get();
			} catch (GridTaskTimeoutException e) {
				throw new DistributionException(e);
			} catch (GridException e) {
				throw new DistributionException(e);
			} catch (Exception e){
				throw new DistributionException(e);
			}
	}

	@Override
	public void provideStaticData(Serializable key, Serializable data) {
		DistributionDataLayer <S> layer = new CoherenceDataLayer <S> (distributionUUID, this.getClass().getClassLoader());
		layer.putData(key, data);
	}

	@Override
	public void provideMapData(int mapNr, Serializable key, Serializable data) {
		DistributionDataLayer <S> layer = new CoherenceDataLayer <S> (distributionUUID, this.getClass().getClassLoader());
		layer.putMapData(mapNr, key, data);
	}

	@Override
	public void provideMapInputData(int mapNr, S data) {
		DistributionDataLayer <S> layer = new CoherenceDataLayer <S> (distributionUUID, this.getClass().getClassLoader());
		layer.putMapInputData(mapNr, data);
	}
	
	public void clearData(){
		DistributionDataLayer <S> layer = new CoherenceDataLayer <S> (distributionUUID, this.getClass().getClassLoader());
		layer.clearData();
	}

	@Override
	public Serializable getStaticData(Serializable key) {
		DistributionDataLayer <S> layer = new CoherenceDataLayer <S> (distributionUUID, this.getClass().getClassLoader());
		return layer.getData(key);
	}

	@Override
	public Serializable getMapData(int mapNr, Serializable key) {
		DistributionDataLayer <S> layer = new CoherenceDataLayer <S> (distributionUUID, this.getClass().getClassLoader());
		return layer.getMapData(mapNr, key);
	}
	
	
	
}
