/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.gridgain;

import java.io.Serializable;
import java.util.UUID;

import org.gridgain.grid.GridException;
import org.gridgain.grid.GridTaskSession;
import org.gridgain.grid.logger.GridLogger;
import org.gridgain.grid.resources.GridLoggerResource;
import org.gridgain.grid.resources.GridTaskSessionResource;
import org.gridgain.grid.spi.loadbalancing.coherence.GridCoherenceAffinityJobAdapter;

import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.distribution.coherence.CoherenceDataLayer;
import de.arimond.dt.operators.distribution.coherence.MapDataKey;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <S>
 * @param <M>
 * @param <R>
 */

class MapJob <S extends Serializable, M extends Serializable, R extends Serializable>extends GridCoherenceAffinityJobAdapter<MapDataKey, Integer>{ // second Integer here not really needed

	/**
	 * 
	 */
	private static final long serialVersionUID = 1582140543295630150L;
    private int mapNr;

	@GridLoggerResource
	private GridLogger logger;
	
	@GridTaskSessionResource
	private GridTaskSession gtsession;
	
	private DistributionMapReduce<S, M, R> spec;
	private UUID distributionUUID;
    
	public MapJob(int i, DistributionMapReduce<S, M, R > spec, UUID distributionUUID) {
		super(CoherenceDataLayer.CACHE_NAME, new MapDataKey(distributionUUID, new Integer(i)));
		this.spec = spec;
		this.distributionUUID = distributionUUID;
		this.mapNr = i;
	}

	@Override
	public M execute() throws GridException{
		try {
			if (spec instanceof GridGainMapReduce<?,?,?>){
				((GridGainMapReduce<S, M, R>) spec).setGridLogger(logger);
			}
			final M mapResult = spec.map(new CoherenceDataLayer <S> (distributionUUID, gtsession.getClassLoader()), mapNr);
			return mapResult;
		} catch (Exception e){
			throw new GridException(e);
		}
	}
	
}