/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.hadoop;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectStreamClass;
import java.io.Serializable;
import java.util.HashMap;

import org.apache.hadoop.filecache.DistributedCache;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.Mapper;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reporter;

import com.rapidminer.operator.clustering.Centroid;

import de.arimond.dt.operators.distribution.DistributionMapReduce;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <S>
 * @param <M>
 * @param <O>
 */

public class DistributionMap <S extends Serializable, M extends Serializable, O extends Serializable> implements Mapper<LongWritable, MapDataObject, LongWritable, TaskOutput> {

	private DistributionMapReduce <S,M,O> spec;
	private Exception e = null;
	private HashMap<Serializable, Serializable> staticData;
	
	@Override
	public void map(LongWritable key, MapDataObject value,
			OutputCollector<LongWritable, TaskOutput> output, Reporter reporter)
			throws IOException {
		
		try {
			Thread.sleep(10000);
		} catch (InterruptedException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		if (e != null) throw new IOException("configuring exception" , e);
		
		HadoopDataLayer <S> layer = new HadoopDataLayer<S>();
		layer.setStaticData(staticData);
		//if (value.getMapNr() == 0) throw new NullPointerException("nr is 0");
		if (value == null) throw new NullPointerException("input to map is null");
		if (value.getMapInputData() == null) throw new NullPointerException("no input data given");
		
		layer.putMapDataObject(value);
		
		// push all values to the same reducer
		try{
			output.collect(new LongWritable(1), new TaskOutput(this.spec.map(layer, value.getMapNr())));
		} catch (Exception e){
			throw new IOException(e);
		}
	}

	@Override
	public void configure(JobConf job) {
		DistributionJobConf <S,M,O> jobConf = new DistributionJobConf<S,M,O> (job);
		try {
			ObjectStreamClass cl = ObjectStreamClass.lookup(Centroid.class);
			if (cl == null) System.out.println("Centroid does not implement serializable");
			//System.out.println("version id:"+  Centroid.serialVersionUID);
			System.out.println(Serializable.class.isAssignableFrom(Centroid.class));
			Centroid c = new Centroid(2);
			System.out.println(c instanceof Serializable);

			
			
//			Centroid c = new Centroid(2);
//			System.out.println(c.getBla());
			
//			Class [] cs = Centroid.class.getInterfaces();
//			for (Class c : cs ){
//				System.out.println(c.getCanonicalName());
//			}
			
			spec = jobConf.getMapReduceSpec();
			Path [] paths = DistributedCache.getLocalCacheFiles(job);
			if (paths != null && paths.length > 0){
				// the files are cached locally on drive, not on hdfs
				ObjectInputStream in = new ObjectInputStream(new FileInputStream(new File(paths[0].toString())));
				staticData = (HashMap<Serializable, Serializable>)in.readObject();
				in.close();
			}
			
		} catch (Exception e){
			this.e = e;
		}
		

		
	}

	@Override
	public void close() throws IOException {
		// TODO Auto-generated method stub
		
	}

}
