/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.hadoop;

import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;

import org.apache.hadoop.filecache.DistributedCache;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.mapred.JobConf;
import org.apache.hadoop.mapred.OutputCollector;
import org.apache.hadoop.mapred.Reducer;
import org.apache.hadoop.mapred.Reporter;

import de.arimond.dt.operators.distribution.DistributionMapReduce;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <S>
 * @param <M>
 * @param <O>
 */

public class DistributionReduce <S extends Serializable, M extends Serializable, O extends Serializable> implements Reducer<LongWritable, TaskOutput, LongWritable, TaskOutput> {

	private DistributionMapReduce<S, M, O> spec;
	private Exception e;
	private HashMap<Serializable, Serializable> staticData;

	@Override
	public void reduce(LongWritable key, Iterator<TaskOutput> values,
			OutputCollector<LongWritable, TaskOutput> output, Reporter reporter)
			throws IOException {
		if (e != null) throw new IOException("configuring exception" , e);

		final LinkedList <M> mapResults = new LinkedList <M> ();
		while(values.hasNext()){
			mapResults.add((M)values.next().getOutput());
		}
		
		HadoopDataLayer <S> layer = new HadoopDataLayer<S>();
		layer.setStaticData(staticData);
		
		try{
			output.collect(new LongWritable(1), new TaskOutput(this.spec.reduce(mapResults, layer)));
		} catch (Exception e){
			throw new IOException(e);
		}
	}

	@Override
	public void configure(JobConf job) {
		DistributionJobConf <S,M,O> jobConf = new DistributionJobConf<S,M,O>(job);
		try {
			spec = jobConf.getMapReduceSpec();
			Path [] paths = DistributedCache.getLocalCacheFiles(job);
			if (paths != null && paths.length > 0){
				ObjectInputStream in = new ObjectInputStream(new FileInputStream(new File(paths[0].toString())));
				staticData = (HashMap<Serializable, Serializable>)in.readObject();
				in.close();
			}
		} catch (Exception e){
			this.e = e;
		}
	}

	@Override
	public void close() throws IOException {
		// TODO Auto-generated method stub
		
	}

	
	
}
