/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.hadoop;

import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.UUID;

import org.apache.hadoop.filecache.DistributedCache;
import org.apache.hadoop.fs.FileSystem;
import org.apache.hadoop.fs.Path;
import org.apache.hadoop.io.LongWritable;
import org.apache.hadoop.io.SequenceFile;
import org.apache.hadoop.io.WritableComparable;
import org.apache.hadoop.io.serializer.JavaSerialization;
import org.apache.hadoop.mapred.JobClient;
import org.apache.hadoop.mapred.SequenceFileInputFormat;
import org.apache.hadoop.mapred.SequenceFileOutputFormat;

import de.arimond.dt.operators.distribution.Distribution;
import de.arimond.dt.operators.distribution.DistributionException;
import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.distribution.gridgain.Grid;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <S>
 * @param <M>
 * @param <O>
 */

public class HadoopDistribution<S extends Serializable, M extends Serializable, O extends Serializable>
		extends Distribution <S,M,O>{

	public HadoopDistribution(DistributionMapReduce<S, M, O> spec) {
		super(spec);		
	}
	
	@Override
	public void clearData() {
		// TODO Auto-generated method stub
	}

	private HashMap<Integer, MapDataObject> mapData = new HashMap<Integer,MapDataObject>();
	private boolean mapDataChanged;

	@Override
	public void provideMapData(int mapNr, Serializable key, Serializable data) {
		mapDataChanged = true;
		MapDataObject mapDataObj;
		if (mapData.containsKey(new Integer(mapNr))){
			mapDataObj = mapData.get(mapNr);
			mapDataObj.setMapData(key, data);
		} else {
			mapDataObj = new MapDataObject(mapNr);
			mapDataObj.setMapData(key, data);
			mapData.put(new Integer(mapNr), mapDataObj);
		}
	}

	@Override
	public void provideMapInputData(int mapNr, S data) {
		mapDataChanged = true;
		MapDataObject mapDataObj;
		if (mapData.containsKey(new Integer(mapNr))){
			mapDataObj = mapData.get(mapNr);
			mapDataObj.setMapInputData(data);
		} else {
			mapDataObj = new MapDataObject(mapNr);
			mapDataObj.setMapInputData(data);
			mapData.put(new Integer(mapNr), mapDataObj);
		}
		
		assert mapData.get(new Integer(mapNr)).getMapInputData() != null;
	}

	private HashMap<Serializable, Serializable> staticData = new HashMap<Serializable, Serializable>();
	private boolean staticDataChanged;

	
	@Override
	public void provideStaticData(Serializable key, Serializable data) {
		staticDataChanged = true;
		staticData.put(key, data);
	}
	
	@Override
	public Serializable getStaticData(Serializable key) {
		//FIXME not functional
		return null;
	}
	
	@Override
	public Serializable getMapData(int mapNr, Serializable key) {
		//FIXME not functional
		return null;
	}

	@Override
	public O startComputation() throws DistributionException {
		try{
			DistributionJobConf <S, M, O> conf = initializeJob();
			if (mapDataChanged) sendMapDataToHDFS(conf);
			if (staticDataChanged) sendStaticDataToHDFS(conf);
			O result = runJob(conf);
			return result;
		} catch(Exception e){
			throw new DistributionException(e);
		}

	}

	private O runJob(DistributionJobConf<S, M, O> conf) throws IOException, DistributionException, InstantiationException, IllegalAccessException {
	
		final boolean deleted = outputPath.getFileSystem(conf).delete(outputPath, true);

		if (deleted) 
			System.out.println("ouput deleted");
		else 
			System.out.println("output not deleted, probably not existed before");
		
		JobClient.runJob(conf);

		FileSystem fs = FileSystem.get(conf);

		//Path result = new Path("/user/a_arimon/tmp/default");
		if (!fs.exists(outputPath)){
			System.out.println("no result!!!!");
			throw new DistributionException("could not find hadoop generated result");
		}
		
		Path outputPart = new Path(outputPath.toString().concat("/part-00000"));
		
		final SequenceFile.Reader in = new SequenceFile.Reader(fs, outputPart, conf);
		WritableComparable key = (WritableComparable) in.getKeyClass().newInstance();
		TaskOutput value = (TaskOutput) in.getValueClass().newInstance();
		in.next(key);
		in.getCurrentValue(value);
		
		
		//final ObjectInputStream in = new ObjectInputStream(fs.open(outputPath));
		//O res = (O) in.readObject();
		in.close();
		return (O)value.getOutput();
	}

	final static String pc = "pc-3401";
	
	final Path inputPath = new Path("hdfs://"+pc+":9000/tmp/input");
	final Path outputPath = new Path("hdfs://"+pc+":9000/tmp/output");
	final Path staticPath = new Path("hdfs://"+pc+":9000/tmp/static");
	
	
	private DistributionJobConf<S, M, O> initializeJob() {
		DistributionJobConf <S,M,O> conf = new DistributionJobConf<S,M,O>();
		
		//FIXME make configurable!
		conf.set("fs.default.name", "hdfs://"+pc+":9000");
		
		conf.set("mapred.job.tracker", pc+":9001");
		
		conf.set("mapred.child.java.opts", "-Xmx1000m");
		
		conf.setMapReduceSpec(this.spec);

		conf.set("io.serializations", conf.get("io.serializations") + "," + JavaSerialization.class.getCanonicalName());
		//conf.set("io.serializations", JavaSerialization.class.getCanonicalName());
		
//		conf.setMapReduceInputPath(inputPath);
//		conf.setMapReduceOutputPath(outputPath);
		
		final String jobName = "rapidminer_" + UUID.randomUUID();
		conf.setJobName(jobName);
		
		System.out.println("hadoop job name: " + jobName);
		
		conf.setOutputKeyClass(LongWritable.class);
		conf.setOutputValueClass(TaskOutput.class);
		
		conf.setMapOutputKeyClass(LongWritable.class);
		conf.setMapOutputValueClass(TaskOutput.class);
		
		conf.setMapperClass(DistributionMap.class);
		conf.setReducerClass(DistributionReduce.class);

		conf.setInputFormat(SequenceFileInputFormat.class);
		conf.setOutputFormat(SequenceFileOutputFormat.class);
				
		conf.set("mapred.input.dir", inputPath.toString());
		conf.set("mapred.output.dir", outputPath.toString());
		
		conf.setNumMapTasks(Grid.nr_threads); // FIXME
		conf.setNumTasksToExecutePerJvm(-1);
		
		return conf;
	}

	private void sendMapDataToHDFS(DistributionJobConf <S, M, O> conf) throws IOException {

		//FileOutputFormat.setOutputPath(conf, outputPath);
		
		FileSystem fs = FileSystem.get(conf);
		
		// write input to hdfs
		final SequenceFile.Writer out = new SequenceFile.Writer(fs, conf, inputPath, LongWritable.class, MapDataObject.class);
		
		LinkedList<MapDataObject> splits = new LinkedList<MapDataObject>(mapData.values());
		int i = 0;
		for (MapDataObject split : splits){
			assert split.getMapInputData() != null;
			out.append(new LongWritable(i), split);
			i++;
		}
		out.close();
		
	}

	private void sendStaticDataToHDFS(DistributionJobConf<S,M,O> conf) throws IOException {
//		final boolean deleted = staticPath.getFileSystem(conf).delete(staticPath, true);
//
//		if (deleted) 
//			System.out.println("static data deleted");
//		else 
//			System.out.println("output not deleted, probably not existed before");

		//FileOutputFormat.setOutputPath(conf, outputPath);
		
		final FileSystem fs = FileSystem.get(conf);
		
		ObjectOutputStream out = new ObjectOutputStream(fs.create(staticPath, true));
		out.writeObject(staticData);
		out.close();
		
		DistributedCache.addCacheFile(staticPath.toUri(), conf);
	}

}
