/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.hadoop;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;
import java.util.HashMap;

import org.apache.hadoop.io.Writable;

/**
 * convenient wrapper
 * @author Alexander Arimond
 *
 */


// must be Writable since normal JavaSerialization does not work within SequenceFile
// there are methods like boolean next (key, value) , which are responsible for deserialization, but dont
// allow to return a deserialized value, therefore it just initializes with empty constructor.
public class MapDataObject implements Writable, Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = 4905659554152896985L;
	private Serializable mapInput;
	private int mapNr;
	
	public MapDataObject(){
		// nothing 
	}
	
	private HashMap<Serializable, Serializable> mapData = new HashMap<Serializable, Serializable>();
	
	public int getMapNr(){
		return this.mapNr;
	}
	
	public MapDataObject(int mapNr){
		this.mapNr = mapNr;
	}
	
	public void setMapInputData(Serializable data){
		this.mapInput = data;
	}
	
	public Serializable getMapInputData(){
		return this.mapInput;
	}
	
	public void setMapData(Serializable key, Serializable value){
		mapData.put(key, value);
	}
	
	public Serializable getMapData(Serializable key){
		return mapData.get(key);
	}

	class DataInputImpl extends InputStream{
		private DataInput in;

		public DataInputImpl(DataInput in){
			super();
			this.in = in;
		}

		@Override
		public int read() throws IOException {
			return in.readUnsignedByte();
		}
	};
	
	class DataOutputImpl extends OutputStream{
		private DataOutput out;

		public DataOutputImpl(DataOutput out){
			super();
			this.out = out;
		}
		
		@Override
		public void write(int b) throws IOException {
			out.writeByte(b);
		}
	};
	
	@Override
	public void readFields(DataInput in) throws IOException {
		final ObjectInputStream objIn = new ObjectInputStream(new DataInputImpl(in))
				{
		        @Override protected void readStreamHeader() {
		          // no header
		        }
		};
		try {
			final MapDataObject obj = (MapDataObject)objIn.readObject();
			this.mapInput = obj.mapInput;
			this.mapNr = obj.mapNr;
			this.mapData = obj.mapData;
		} catch (ClassNotFoundException e) {
			throw new IOException(e);
		} finally {
			objIn.close();
		}
		
	}

	// FIXME maybe use BytesWritable instead of MapDataObject etc.
	
	@Override
	public void write(DataOutput out) throws IOException {
		final ObjectOutputStream objectOut = new ObjectOutputStream(new DataOutputImpl(out)){
	        @Override protected void writeStreamHeader() {
	            // no header
	          }
	    };
		objectOut.writeObject(this);
		objectOut.close();
	}
	
}
