/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.hadoop;

import java.io.DataInput;
import java.io.DataOutput;
import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;
import java.io.Serializable;

import org.apache.hadoop.io.Writable;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class TaskOutput implements Writable, Serializable{

	/**
	 * 
	 */
	private static final long serialVersionUID = -2127734859894710690L;
	private Serializable output;

	public TaskOutput(){
		
	}
	
	public TaskOutput(Serializable output){
		this.output = output;
	}
	
	public Serializable getOutput(){
		return output;
	}
	
	class DataInputImpl extends InputStream{
		private DataInput in;

		public DataInputImpl(DataInput in){
			super();
			this.in = in;
		}

		@Override
		public int read() throws IOException {
			return in.readUnsignedByte();
		}
	};
	
	class DataOutputImpl extends OutputStream{
		private DataOutput out;

		public DataOutputImpl(DataOutput out){
			super();
			this.out = out;
		}
		
		@Override
		public void write(int b) throws IOException {
			out.writeByte(b);
		}
	};
	
	@Override
	public void readFields(DataInput in) throws IOException {
		final ObjectInputStream objIn = new ObjectInputStream(new DataInputImpl(in)){
			@Override protected void readStreamHeader() {
				// no header
			}
		};
		try {
			final TaskOutput obj = (TaskOutput)objIn.readObject();
			this.output = obj.getOutput();
		} catch (ClassNotFoundException e) {
			throw new IOException(e);
		} finally {
			objIn.close();
		}
		
	}

	@Override
	public void write(DataOutput out) throws IOException {
		final ObjectOutputStream objectOut = new ObjectOutputStream(new DataOutputImpl(out)){
	        @Override protected void writeStreamHeader() {
	            // no header
	          }
	    };
		objectOut.writeObject(this);
		objectOut.close();
	}
	
	
}
