/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.images;

import java.util.List;

import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.GenerateNewExampleSetMDRule;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;

import de.arimond.dt.features.Features;
import de.arimond.dt.operators.distribution.AbstractDistributableOperator;
import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.images.ImageObject;
import de.arimond.dt.operators.images.ImageObjectList;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <F>
 */

public abstract class ImageFeatureExtractionOperator <F extends Features> extends AbstractDistributableOperator<ImageObjectList, ImageObject, F, ExampleSet> {

	private final InputPort inputPort = getInputPorts().createPort("input", ImageObjectList.class);
	private final OutputPort outputPort = getOutputPorts().createPort("features");
	private final OutputPort original = getOutputPorts().createPort("original");
	
	public ImageFeatureExtractionOperator(OperatorDescription description) {
		super(description);
		inputPort.addPrecondition(new SimplePrecondition(inputPort, new MetaData(ImageObjectList.class), true));
		GenerateNewExampleSetMDRule rule = new GenerateNewExampleSetMDRule(outputPort){
			public MetaData modifyMetaData(ExampleSetMetaData unmodifiedMD){
				return ImageFeatureExtractionOperator.this.getImageFeatureExtractionSpec().getResultMetaData(unmodifiedMD);
			}
		};
		getTransformer().addRule(rule);
		getTransformer().addPassThroughRule(inputPort, original);
	}	
	
	protected abstract ImageFeatureExtractionSpec <F> getImageFeatureExtractionSpec();

	@Override
	protected DistributionMapReduce<ImageObject, F, ExampleSet> getMapReduceSpecification() {
		return getImageFeatureExtractionSpec();
	}
	
	@Override
	protected ImageObjectList getDistributionInput() throws OperatorException {
		return (ImageObjectList) inputPort.getData();
	}
	
	@Override
	protected void deliverDistributionOutput(ExampleSet output) {
		outputPort.deliver(output);
	}
	
	@Override
	public List<ImageObject> split(ImageObjectList input) {
		return input.getList();
	}
}
