/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.images;

import java.util.LinkedList;
import java.util.List;

import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.GenerateNewExampleSetMDRule;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.SimplePrecondition;
import com.rapidminer.parameter.ParameterType;
import com.rapidminer.parameter.ParameterTypeInt;
import com.rapidminer.parameter.UndefinedParameterError;

import de.arimond.dt.features.Features;
import de.arimond.dt.operators.distribution.AbstractDistributableOperator;
import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.images.ImageObject;
import de.arimond.dt.operators.images.ImageObjectList;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <F>
 */

public abstract class ImageFeatureExtractionOperator1 <F extends Features> extends AbstractDistributableOperator<ImageObjectList, ImageObjectList, LinkedList<F>, ExampleSet> {

	private final InputPort inputPort = getInputPorts().createPort("input", ImageObjectList.class);
	private final OutputPort outputPort = getOutputPorts().createPort("features");
	private final OutputPort original = getOutputPorts().createPort("original");
	
	public ImageFeatureExtractionOperator1(OperatorDescription description) {
		super(description);
		inputPort.addPrecondition(new SimplePrecondition(inputPort, new MetaData(ImageObjectList.class), true));
		GenerateNewExampleSetMDRule rule = new GenerateNewExampleSetMDRule(outputPort){
			public MetaData modifyMetaData(ExampleSetMetaData unmodifiedMD){
				return ImageFeatureExtractionOperator1.this.getImageFeatureExtractionSpec().getResultMetaData(unmodifiedMD);
			}
		};
		getTransformer().addRule(rule);
		getTransformer().addPassThroughRule(inputPort, original);
	}	
	
	protected abstract ImageFeatureExtractionSpec1 <F> getImageFeatureExtractionSpec();

	@Override
	protected DistributionMapReduce<ImageObjectList, LinkedList<F>, ExampleSet> getMapReduceSpecification() {
		return getImageFeatureExtractionSpec();
	}
	
	@Override
	protected ImageObjectList getDistributionInput() throws OperatorException {
		return (ImageObjectList) inputPort.getData();
	}
	
	@Override
	protected void deliverDistributionOutput(ExampleSet output) {
		outputPort.deliver(output);
	}
	
	@Override
	public List<ImageObjectList> split(ImageObjectList input) {
	
		LinkedList<ImageObjectList> splitted = new LinkedList<ImageObjectList> ();
		
		// FIXME very rudimentary
		int size = 1;
		try {
			size = getParameterAsInt(PARAMETER_SPLIT_SIZE);
		} catch (UndefinedParameterError e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}		

		
		int index = 0;
		boolean reachedEnd = false;
		while(!reachedEnd){
			int start = index * size;
			int end = Math.min(start + size, input.getList().size());
			splitted.add(copySubList(input.getList().subList(start, end)));
			
			index ++;
			if (end == input.getList().size()) reachedEnd = true;
		}
		
		for (ImageObjectList l : splitted){
			if (l.getList().isEmpty()){
				splitted.remove(l);
			}
		}
		
		return splitted;
	}

	// this must be done because subList() return java.util.Sublist which is not Serializable
	private ImageObjectList copySubList(List <ImageObject> list){
		LinkedList <ImageObject> copiedList = new LinkedList<ImageObject>();
		for (ImageObject image : list )
			copiedList.add(image);
		return new ImageObjectList(copiedList);
	}
	
	
	public static String PARAMETER_SPLIT_SIZE = "split_size";
	
	@Override
	public List<ParameterType> getParameterTypes() {
		List <ParameterType> types = super.getParameterTypes();
		ParameterTypeInt type = new ParameterTypeInt(PARAMETER_SPLIT_SIZE, "number of images in one job", 0, 1500, 1);
		types.add(type);
		return types;
	}
	
}
