/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.images;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.example.table.AttributeFactory;
import com.rapidminer.example.table.DoubleArrayDataRow;
import com.rapidminer.example.table.MemoryExampleTable;
import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.tools.Ontology;
import com.rapidminer.tools.att.AttributeSet;

import de.arimond.dt.features.Feature;
import de.arimond.dt.features.Features;
import de.arimond.dt.operators.distribution.DistributionDataLayer;
import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.images.ImageObject;
import de.arimond.dt.operators.images.ImageObjectList;

/**
 * 
 * @author Alexander Arimond
 *
 * @param <F>
 */

public abstract class ImageFeatureExtractionSpec1 <F extends Features> extends
		DistributionMapReduce<ImageObjectList, LinkedList<F> , ExampleSet> {

	/**
	 * 
	 */
	private static final long serialVersionUID = -8153370145891270380L;

	@Override
	public LinkedList<F> map(DistributionDataLayer<ImageObjectList> dataLayer, int mapNr)
			throws Exception {
		ImageObjectList images = dataLayer.getMapInputData(mapNr);
		LinkedList<F> features = new LinkedList<F>();
		for(ImageObject image : images.getList()){
			features.add(this.extractFeatures(image));
		}
		return features;
	}

	protected abstract F extractFeatures(ImageObject mapInputData) throws FeatureExtractionException;

	@Override
	public ExampleSet reduce(List<LinkedList<F>> mapResults,
			DistributionDataLayer<ImageObjectList> dataLayer) throws Exception {
		final int dim = 128;
		AttributeSet set = new AttributeSet();
        final Attribute idAttribute = AttributeFactory.createAttribute("filename", Ontology.NOMINAL);
        set.addAttribute(idAttribute);
        for (int i = 0; i < dim; i++) {
            set.addAttribute(AttributeFactory.createAttribute("att" + i, Ontology.NUMERICAL));
        }

        // create table
        final MemoryExampleTable table = new MemoryExampleTable(set.getAllAttributes());
        for (LinkedList<F> list : mapResults){
        	for (F features : list){
        		//F features = values.next();
        		Iterator <Feature> iter = features.iterator();
        		while (iter.hasNext()){
        			Feature feat = iter.next();
        			final double[] dataRow = new double[1 + dim];

        			dataRow[0] = idAttribute.getMapping().mapString(features.getID());
        			for (int j = 1; j < dim + 1; j++){
        				dataRow[j] = feat.values[j -1];
        			}
        			table.addDataRow(new DoubleArrayDataRow(dataRow));
        		}
        	}
        }
        final ExampleSet resultSet = table.createExampleSet(set);
        resultSet.getAttributes().setId(idAttribute);
        return resultSet;
	}

	public abstract MetaData getResultMetaData(ExampleSetMetaData newlyCreated);

}
