/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.distribution.images;

import java.util.List;

import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;

import de.arimond.dt.operators.distribution.AbstractDistributableOperator;
import de.arimond.dt.operators.distribution.DistributionMapReduce;
import de.arimond.dt.operators.images.ImageObject;
import de.arimond.dt.operators.images.ImageObjectList;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class SimpleImageDistributionOperator extends AbstractDistributableOperator<ImageObjectList, ImageObject, ImageObject, ImageObjectList>{

	private final InputPort inputPort = getInputPorts().createPort("input", ImageObjectList.class);
	private final OutputPort outputPort = getOutputPorts().createPort("output");
	
	public SimpleImageDistributionOperator(OperatorDescription description) {
		super(description);
	}


	@Override
	protected ImageObjectList getDistributionInput() throws OperatorException {
		return (ImageObjectList) inputPort.getData();
	}


	@Override
	public List<ImageObject> split(ImageObjectList input) {
		return input.getList();
	}


	@Override
	protected void deliverDistributionOutput(ImageObjectList output) {
		outputPort.deliver(output);
	}


	@Override
	protected DistributionMapReduce<ImageObject, ImageObject, ImageObjectList> getMapReduceSpecification() {
		return new SimpleImageDistributionSpec();
	}

}
