/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.features.extraction;

import java.io.File;

import com.rapidminer.operator.ports.metadata.ExampleSetMetaData;
import com.rapidminer.operator.ports.metadata.MetaData;

import de.arimond.dt.features.SURFFeatures;
import de.arimond.dt.features.extraction.SURFFeatureExtractor;
import de.arimond.dt.operators.distribution.images.FeatureExtractionException;
import de.arimond.dt.operators.distribution.images.ImageFeatureExtractionSpec1;
import de.arimond.dt.operators.images.ImageObject;
import de.arimond.dt.preprocessing.GrayScaleImageConverter;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class SURFExtractionSpec1 extends ImageFeatureExtractionSpec1<SURFFeatures>{

	/**
	 * 
	 */
	private static final long serialVersionUID = 7660617827672441854L;

	public static File workingDirectory = new File("/tmp/" +System.getProperty("user.name") + "_feature_extraction");
	
	@Override
	protected SURFFeatures extractFeatures(ImageObject image) throws FeatureExtractionException {
		if (!workingDirectory.exists()){
			workingDirectory.mkdir();
		}
		final File imageFile = new File(image.getFilename());
		final File grayImage = new File(workingDirectory, imageFile.getName() + ".gray.pgm");
		try {
			try{
				// create gray image
				GrayScaleImageConverter.grayImage(imageFile, grayImage, "pgm");
			} catch (NoClassDefFoundError err){
				System.err.println("class not found, maybe JAI is not in classpath");
				throw err;
			}
			// extract features
			SURFFeatures feats = SURFFeatureExtractor.extractFeatures(grayImage, workingDirectory);
			feats.setID(image.getFilename());
			return feats;
		} catch (Exception e){
			throw new FeatureExtractionException(e);
		} finally {
			grayImage.delete();
		}
	}

	@Override
	public MetaData getResultMetaData(ExampleSetMetaData newlyCreated) {
		// TODO
		return newlyCreated;
	}

	
	
}