/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.images;

import java.util.Iterator;
import java.util.LinkedList;

import com.rapidminer.example.Attribute;
import com.rapidminer.example.Attributes;
import com.rapidminer.example.Example;
import com.rapidminer.example.ExampleSet;
import com.rapidminer.operator.Operator;
import com.rapidminer.operator.OperatorDescription;
import com.rapidminer.operator.OperatorException;
import com.rapidminer.operator.ports.InputPort;
import com.rapidminer.operator.ports.OutputPort;
import com.rapidminer.operator.ports.metadata.ExampleSetPrecondition;
import com.rapidminer.operator.ports.metadata.GenerateNewMDRule;
import com.rapidminer.operator.ports.metadata.MetaData;
import com.rapidminer.operator.ports.metadata.Precondition;
import com.rapidminer.tools.Ontology;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class GetImagesOperator extends Operator {
	
    //private final InputPortExtender inputPortExtender = new InputPortExtender("images", getInputPorts());
	private final InputPort filenameInputPort = getInputPorts().createPort("filenames");
	private final OutputPort imageOutputPort = getOutputPorts().createPort("images");
	
	/*
	public class FilenameListPrecondition extends SimplePrecondition{

		public FilenameListPrecondition(InputPort inputPort,
				MetaData desiredMetaData, boolean mandatory) {
			super(inputPort, desiredMetaData, mandatory);
		}
		
		public void makeAdditionalChecks(MetaData received) {
			if (!(received instanceof ExampleSetMetaData)){
				
			}
		}
		
	}
	*/
	
	public GetImagesOperator(OperatorDescription description) {
		super(description);
		this.filenameInputPort.addPrecondition(getPreconditon());
		getTransformer().addRule(new GenerateNewMDRule(imageOutputPort, new MetaData(ImageObjectList.class)));
		//inputPortExtender.ensureMinimumNumberOfPorts(1);
		//inputPortExtender.start();
		//this.imageOutputPort.deliverMD(new MetaData(ImageObjectList.class));
		// Precondition cond = new FilenamePrecondition(filenameInputPort, requiredMetaData) extends SimplePreconditon {
	}
	
	protected Precondition getPreconditon(){
		return new ExampleSetPrecondition(filenameInputPort, new String [] {"filename"}, Ontology.ATTRIBUTE_VALUE, Attributes.LABEL_NAME, Attributes.ID_NAME);
		//return new ExampleSetPrecondition(filenameInputPort, new String [] {"filename"}, Ontology.ATTRIBUTE_VALUE, Ontology.ATTRIBUTE_VALUE, "label", "id");
	}
	
	/*
	protected MetaData getRequiredMetaData(){
		ExampleSetMetaData md = new ExampleSetMetaData();
		md.addAttribute(new AttributeMetaData("filename", Ontology.NOMINAL));
		md.addAttribute(new AttributeMetaData("label", Ontology.N));
	}
	*/

	public void doWork() throws OperatorException{
		final ImageObjectList list = new ImageObjectList(new LinkedList <ImageObject> ());
		final ExampleSet filenameSet = filenameInputPort.getData();
		final Attribute attr = filenameSet.getAttributes().get("filename");
		final Attribute labelAttr = filenameSet.getAttributes().getLabel();
		Iterator <Example> iter = filenameSet.iterator();
		while (iter.hasNext()){
			final Example row = iter.next();
			String filename = row.getValueAsString(attr);
			labelAttr.getValueType();
			String label = row.getValueAsString(labelAttr);
			list.getList().add(new ImageObject(filename, filename, label));
		}
		list.setSource(this.getName());
		imageOutputPort.deliver(list);
	}
	
	
	
}
