/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.images;

import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;

import com.rapidminer.gui.renderer.RendererService;
import com.rapidminer.operator.ResultObjectAdapter;

/**
 * @author Alexander Arimond
 */

public class ImageObjectList extends ResultObjectAdapter {

	private static int MAX_FILENAMES_IN_RESULT = 16;
	
	private static boolean called = false;
	private static synchronized void registerRenderer(){
		if (called) return;
		called = true;
		final String name = "Image Object List";
		final String className = ImageObjectList.class.getCanonicalName();
		final String superTypeName = ImageObjectList.class.getSuperclass().getCanonicalName();
		final boolean reportable = true;
		final List<String> rendererNames = new LinkedList<String>();
		rendererNames.add(ImageObjectListRenderer.class.getCanonicalName());
		ClassLoader classLoader = ImageObjectList.class.getClassLoader();
		RendererService.registerRenderers(superTypeName, className, reportable, rendererNames, classLoader);
		//RendererService.registerRenderers(name, className, superTypeName, reportable, rendererNames, classLoader);
	}
	
	private List<ImageObject> imageObjectList;

	public ImageObjectList (List <ImageObject> imageObjectList){
		this.imageObjectList = imageObjectList;
		ImageObjectList.registerRenderer();
	}
	
	public List <ImageObject> getList(){
		return this.imageObjectList;
	}

//	@Override
//	public String getExtension() {
//		// TODO Auto-generated method stub
//		return "img";
//	}
//
//	@Override
//	public String getFileDescription() {
//		// TODO Auto-generated method stub
//		return "bla bla";
//	}

	public String toResultString() {
		//return "<img src=\"file:///home2/a_arimon/hello.jpg\"/>'";
		
		Iterator <ImageObject> iter = imageObjectList.iterator();
		String list;
		if (iter.hasNext())
			list = iter.next().getFilename();
		else return "";
		int i = 0 ;
		while (iter.hasNext() &&  i<MAX_FILENAMES_IN_RESULT){
			list = list + "\n" + iter.next().getFilename();
			i++;
		}
		if (iter.hasNext()){
			final int rest = imageObjectList.size() - MAX_FILENAMES_IN_RESULT;
			list = list + "\n" + rest + " more.."; 
		}
		return list;
	}
	
	public String toString(){
		return toResultString();
	}
	
	public String [] getFileNames(){
		LinkedList <String> filenames = new LinkedList<String>();
		for (ImageObject image : this.imageObjectList){
			filenames.add(image.getFilename());
		}
		return filenames.toArray(new String[]{});
	}
	
	public String [] getLabels(){
        LinkedList <String> labelList = new LinkedList <String> ();
        for (ImageObject image: this.getList()){
        	if (!labelList.contains(image.getLabel())){
        		labelList.add(image.getLabel());
        	}
        }
        return labelList.toArray(new String[]{});
	}
	
	public List <ImageObject> getImagesWithLabel(String label){
		final LinkedList <ImageObject> list = new LinkedList <ImageObject> ();
		for (ImageObject image : this.getList()){
			if (image.getLabel().equals(label)){
				list.add(image);
			}
		}
		return list;
	}
	
}
