/*
 *  RapidMiner DisPaRe
 *
 *  Copyright (C) 2009-2010 by Deutsches Forschungszentrum fuer
 *  Kuenstliche Intelligenz GmbH or its licensors, as applicable.
 *  
 *  This is free software: you can redistribute it and/or modify 
 *  it under the terms of the GNU Affero General Public License as published by 
 *  the Free Software Foundation, either version 3 of the License, or 
 *  (at your option) any later version. 
 *  
 *  You should have received a copy of the GNU Affero General Public License 
 *  along with this software. If not, see <http://www.gnu.org/licenses/. 
 */
package de.arimond.dt.operators.images;

import java.awt.Component;

import javax.swing.JEditorPane;
import javax.swing.JLabel;

import com.rapidminer.gui.renderer.DefaultTextRenderer;
import com.rapidminer.gui.tools.ExtendedJScrollPane;
import com.rapidminer.operator.IOContainer;
import com.rapidminer.operator.ResultObject;
import com.rapidminer.tools.Tools;

/**
 * 
 * @author Alexander Arimond
 *
 */

public class ImageObjectListRenderer extends DefaultTextRenderer {

	public Component getVisualizationComponent(Object renderable, IOContainer ioContainer) {
		JEditorPane resultText = new JEditorPane();
		resultText.setContentType("text/html");
		resultText.setBorder(javax.swing.BorderFactory.createEmptyBorder(11, 11, 11, 11));
		resultText.setEditable(false);
		resultText.setBackground((new JLabel()).getBackground());

		if (renderable instanceof ImageObjectList){
			String str = toHTMLWithImgTags((ImageObjectList)renderable);
			resultText.setText("<html><h1>" + renderable.getClass().getSimpleName() + "</h1>" + str + "</html>");
		} else if (renderable instanceof ResultObject) {
			ResultObject result = (ResultObject)renderable;
			String str = toHTML(result.toResultString());
			resultText.setText("<html><h1>" + result.getName() + "</h1><pre>" + str + "</pre></html>");			 
		} else {
			String str = toHTML(renderable.toString());
			resultText.setText("<html><h1>" + renderable.getClass().getSimpleName() + "</h1><pre>" + str + "</pre></html>");
		}
		return new ExtendedJScrollPane(resultText);
	}

	private String toHTML(String string) {
		String str = string;
		str = str.replaceAll(">", "&gt;");
		str = str.replaceAll("<", "&lt;");
		str = str.replaceAll(Tools.getLineSeparator(), "<br>");
		return str;
	}
	
	private String toHTMLWithImgTags(ImageObjectList imageObjectList) {
		String htmlSource = "";
		final String [] labels =imageObjectList.getLabels();
		for (int i=0; i<labels.length; i++){
			htmlSource = htmlSource + "<h3>Label : " + labels[i] + "</h3>";
			for (ImageObject image : imageObjectList.getImagesWithLabel(labels[i])){
				htmlSource = htmlSource + "<img src=\"file://" + image.getFilename() + "\" width=\"80\" height=\"80\"/>";
			}
		}
		return htmlSource;
	}

}
