package de.arimond.dt.test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.io.File;
import java.util.Iterator;

import org.junit.Before;
import org.junit.Test;

import de.arimond.dt.features.SURFFeatures;
import de.arimond.dt.features.SURFFeatures.SURFFeature;
import de.arimond.dt.features.extraction.SURFFeatureExtractor;
import de.arimond.dt.preprocessing.GrayScaleImageConverter;

public class SURFFeatureExtractionTest {

	private File testImage = new File("/ds/videos/youtube-densetags/dataset/talkshow/bKE-UUddDTc/bKE-UUddDTc_000017_002219.jpg");
	
	@Before
	public void setUp() throws Exception {
		File workingDir = new File("workingDir");
		if (!workingDir.exists()) {
			workingDir.mkdir();
		}
	}
	
	/*
	@After
	public void tearDown() throws Exception {
		File workingDir = new File("workingDir");
		if (workingDir.exists()){
			File [] list = workingDir.listFiles();
			for (int i=0; i<list.length; i++){
				list[i].delete();
			}
			workingDir.delete();
		}
	}
    */
	@Test
	public void testExtractFeatures() {
		// working dir should exist here because of setUp()
		File workingDir = new File("workingDir");
		// this is tested below.
		File dest = new File("workingDir/testImage.pgm");

		
        try {
    		GrayScaleImageConverter.grayImage(testImage , dest , "pgm");
			SURFFeatures feats = SURFFeatureExtractor.extractFeatures(dest, workingDir);
			Iterator <SURFFeature> iter = feats.iterator();
			while(iter.hasNext()){
				SURFFeature feat = iter.next();
				assertEquals(128, feat.values.length);
				//System.out.println("" +feat.x + " " + feat.y);
			}
        } catch (Exception e){
			e.printStackTrace();
			fail(e.getMessage());
		}

	}


//	public void testDetectRegions(){
//		// working dir should exist here because of setUp()
//		File workingDir = new File("workingDir");
//		// this is tested below.
//		File dest = new File("workingDir/testImage.pgm");
//
//		
//        try {
//    		GrayScaleImageConverter.grayImage(testImage , dest , "pgm");
//			SURFFeatureExtractor.detectRegions(dest, workingDir);
//		} catch (Exception e){
//			e.printStackTrace();
//			fail(e.getMessage());
//		}
//	}

	@Test
	public void testConvertToGrayscale() {
		File dest = new File("workingDir/testImage.pgm");
		try {
			GrayScaleImageConverter.grayImage(testImage , dest, "pgm");
		} catch (Exception e) {
			e.printStackTrace();
			fail(e.getMessage());
		}
	}
	
}
